
//***************************************************************************
//===========================================================================
//---------------------------------------------------------------------------
//
//                    Simple Graphic Library 3D 
//                          << SGL3d >>
//
//                       by Takashi Taniguchi
//                       2001. July. 15 (My Birthday) Version 0.1
//
//---------------------------------------------------------------------------
//===========================================================================
//***************************************************************************
#include <GL/gl.h>
#include <GL/glu.h>
#include <GL/glx.h>
#include <GL/glut.h>
#include <math.h>
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#define _MESH   15
#define _WIDTH  1300
#define _HEIGHT 1000

int    window = 1;
double sgl3d_delta_spin=0.0;
double sgl3d_near_view_volume=2.0;
double sgl3d_far_view_volume=3000.0;
double sgl3d_view_length;
double sgl3d_view_position_x=0.0;
double sgl3d_view_position_y=0.0;
double sgl3d_view_position_z=0.0;

//===========================================================================
//                           Objects
//===========================================================================
void sgl3d_sphere(double x,double y,double z,double radius)
{
  glPushMatrix();
  glTranslatef(x,y,z);
  glutSolidSphere(radius,_MESH,_MESH);
  glPopMatrix();
}
//===========================================================================
void sgl3d_wiresphere(double x,double y,double z,double radius)
{
  glPushMatrix();
  glTranslatef(x,y,z);
  glutWireSphere(radius,_MESH,_MESH);
  glPopMatrix();
}

//===========================================================================
void sgl3d_cylinder(double x1,double y1,double z1,
		    double x2,double y2,double z2,double radius)
{
  double theta;
  double distance;
  double delta_x;
  double delta_y;
  double delta_z;
  GLUquadricObj *tessobj;

  delta_x=x2-x1;
  delta_y=y2-y1;
  delta_z=z2-z1;
  distance=sqrt(delta_x*delta_x+delta_y*delta_y+delta_z*delta_z);
  theta=acos(delta_z/distance);

  tessobj=gluNewQuadric();
  glPushMatrix();
  glTranslatef(x1,y1,z1);
  if(delta_x==0  && delta_y==0){
    glRotatef(theta*180/M_PI,1,0,0);
  }else{
    glRotatef(theta*180/M_PI,-delta_y,delta_x,0.0);
  }
  gluCylinder(tessobj,radius,radius,distance,_MESH,_MESH);
  glPopMatrix();
}

//===========================================================================
void sgl3d_cube(double x,double y,double z,double length)
{
  glPushMatrix();
  glTranslatef(x,y,z);
  glutSolidCube(length);
  glPopMatrix();
}

//===========================================================================
void sgl3d_wirecube(double x,double y,double z,double length)
{
  glPushMatrix();
  glTranslatef(x,y,z);
  glutWireCube(length);
  glPopMatrix();
}

//===========================================================================
void sgl3d_box(double x,double y,double z,
	       double length_x,double length_y,double length_z)
{
  double rate_y_x;
  double rate_z_x;
  rate_y_x=length_y/length_x;
  rate_z_x=length_z/length_x;
  glPushMatrix();
  glTranslatef(x,y,z);
  glScalef(1.0,rate_y_x,rate_z_x);
  glutSolidCube(length_x);
  glPopMatrix();
}

//===========================================================================
void sgl3d_wirebox( double x,double y,double z,
		    double length_x,double length_y,double length_z)
{
  double rate_y_x;
  double rate_z_x;
  rate_y_x=length_y/length_x;
  rate_z_x=length_z/length_x;
  glPushMatrix();
  glTranslatef(x,y,z);
  glScalef(1.0,rate_y_x,rate_z_x);
  glutWireCube(length_x);
  glPopMatrix();
}

//===========================================================================
void sgl3d_line(double x1,double y1,double z1,
		double x2,double y2,double z2, double line_width)
{
  glLineWidth(line_width);
  glBegin(GL_LINES);
  glVertex3f(x1,y1,z1);
  glVertex3f(x2,y2,z2);
  glEnd();
}
//===========================================================================
void sgl3d_parallelogram(double x1,double y1,double z1,
			 double x2,double y2,double z2,
			 double x3,double y3,double z3)
{
  double x4,y4,z4;
  x4=x3+x1-x2;
  y4=y3+y1-y2;
  z4=z3+z1-z2;
  glPolygonMode(GL_FRONT_AND_BACK,GL_FILL);
  glBegin(GL_QUADS);
  glVertex3f(x1,y1,z1);
  glVertex3f(x2,y2,z2);
  glVertex3f(x3,y3,z3);
  glVertex3f(x4,y4,z4);
  glEnd();
}

//===========================================================================
void sgl3d_triangle(double x1,double y1,double z1,
		    double x2,double y2,double z2,
		    double x3,double y3,double z3)
{
  glPolygonMode(GL_FRONT_AND_BACK,GL_FILL);
  glBegin(GL_TRIANGLES);
  glVertex3f(x1,y1,z1);
  glVertex3f(x2,y2,z2);
  glVertex3f(x3,y3,z3);
  glEnd();
}
//===========================================================================
void sgl3d_tetrahedron(double x1,double y1,double z1,
		       double x2,double y2,double z2,
		       double x3,double y3,double z3
		       ,double x4,double y4,double z4)
{
  sgl3d_triangle(x1,y1,z1,x2,y2,z2,x3,y3,z3);
  sgl3d_triangle(x1,y1,z1,x2,y2,z2,x4,y4,z4);
  sgl3d_triangle(x2,y2,z2,x3,y3,z3,x4,y4,z4);
  sgl3d_triangle(x1,y1,z1,x3,y3,z3,x4,y4,z4);
}
//===========================================================================
void sgl3d_wiretetrahedron(double x1,double y1,double z1,
			   double x2,double y2,double z2,
			   double x3,double y3,double z3,
			   double x4,double y4,double z4)
{
  sgl3d_line(x1,y1,z1,x2,y2,z2,1.0);
  sgl3d_line(x1,y1,z1,x3,y3,z3,1.0);
  sgl3d_line(x1,y1,z1,x4,y4,z4,1.0);
  sgl3d_line(x2,y2,z2,x3,y3,z3,1.0);
  sgl3d_line(x3,y3,z3,x4,y4,z4,1.0);
  sgl3d_line(x2,y2,z2,x4,y4,z4,1.0);
}
//============================================================================
//                    << Character Object >>
//============================================================================
/*
void sgl3d_character2(char *charcter,double x,double y,GLfloat *Colorname)
{
  char *p;

  glPushMatrix();
  glPushAttrib(GL_CURRENT_BIT);
  glDisable(GL_LIGHTING);
  glLoadIdentity();
  glColor3fv(Colorname);
  glRasterPos3f(x,y,-sgl3d_view_length*sgl3d_near_view_volume);
  for(p= charcter;*p;p++){
    glutBitmapCharacter(GLUT_BITMAP_9_BY_15,*p);
  }
  glPopAttrib();
  glEnable(GL_LIGHTING);
  glPopMatrix();
}
*/
//============================================================================
void sgl3d_character( char *charcter,double x,double y,double z,
		       GLfloat *Colorname)
{
  char *p;
  glPushMatrix();
  glPushAttrib(GL_CURRENT_BIT);
  glDisable(GL_LIGHTING);
  /*	glLoadIdentity();*/
  glColor3fv(Colorname);
  glRasterPos3f(x,y,z);
  for(p= charcter;*p;p++){
    glutBitmapCharacter(GLUT_BITMAP_9_BY_15,*p);
  }
  glPopAttrib();
  glEnable(GL_LIGHTING);
  glPopMatrix();
}
//============================================================================
/*
void sgl3d_number(double number,double x,double y,GLfloat *Colorname)
{
  char *p;
  char char_number[10];
  sprintf(char_number,"%f",number);

  glPushMatrix();
  glPushAttrib(GL_CURRENT_BIT);
  glDisable(GL_LIGHTING);
  glLoadIdentity();
  glColor3fv(Colorname);
  glRasterPos3f(x,y,-sgl3d_view_length*sgl3d_near_view_volume);
  for(p= char_number;*p;p++){
    glutBitmapCharacter(GLUT_BITMAP_9_BY_15,*p);
  }
  glPopAttrib();
  glEnable(GL_LIGHTING);
  glPopMatrix();
}
*/
//============================================================================
//                            Dump of Window
//===========================================================================
void sgl3d_window2ppm(char *file_name,int number)
{
  char command1[100];
  char command2[100];
  char char_number[6];
  char rm[20]="rm ";

  sprintf(char_number,"%d",number);

  // /usr/bin/X11/xwd -out file_name(number).xwd -name OpenGL  ***/
  strcpy(command1,"/usr/X11R6/bin/xwd -silent -out ");
  strcat(command1,file_name);
  strcat(command1,char_number);
  strcat(command1,".xwd -name OpenGL");
  system(command1);
	
  // /usr/local/bin/convert  file_name(number).xwd file_name(number).ppm ***/

  //strcpy(command2,"/usr/bin/convert ");
  strcpy(command2,"convert ");
  strcat(command2,file_name);
  strcat(command2,char_number);
  strcat(command2,".xwd ");
  strcat(command2,file_name);
  strcat(command2,char_number);
  strcat(command2,".ppm");
  system(command2);

  //  rm file_name(number).xwd 

  strcat(rm,file_name);
  strcat(rm,char_number);
  strcat(rm,".xwd");
  system(rm);
}

//============================================================================
void sgl3d_window2xwd(char *file_name,int number)
{
  char command1[100];
  char char_number[6];

  sprintf(char_number,"%.4d",number);

	//  /usr/bin/X11/xwd -out file_name(number).xwd -name OpenGL  //
  strcpy(command1,"/usr/X11R6/bin/xwd -silent -out ");
  strcat(command1,file_name);
  strcat(command1,char_number);
  strcat(command1,".xwd -name OpenGL");
  system(command1);
}

//===========================================================================
//                   <<  Control Window >>
//===========================================================================

void sgl3d_init( void )
{
  GLfloat light_position[] = {0.4, 0.7, 0.9, 0.0};
  GLfloat light_ambient [] = {0.2, 0.2, 0.2, 1.0};
  GLfloat light_diffuse [] = {1.0, 1.0, 1.0, 1.0};

  /*******  lighting  *********/
  glLightfv(GL_LIGHT0, GL_POSITION, light_position);// light position
  glLightfv(GL_LIGHT0, GL_AMBIENT,  light_ambient);// Environment light
  glLightfv(GL_LIGHT0, GL_DIFFUSE,  light_diffuse);// Diffuse     light

  glEnable(GL_LIGHTING);
  glEnable(GL_LIGHT0);
  glDepthFunc(GL_LEQUAL);// Draw Front Objects
  glEnable(GL_DEPTH_TEST);

  /****** using ALPHA  *******/
  glEnable(GL_BLEND);
  glEnable(GL_ALPHA_TEST);
  glAlphaFunc(GL_GREATER,0.0);
  glBlendFunc(GL_SRC_ALPHA,GL_ONE_MINUS_SRC_ALPHA);
  /** Drawing smooth picture **/
  glShadeModel(GL_SMOOTH);   
}
//============================================================================
void sgl3d_reshape( int width, int height )
{
  float ratio = (float) width / (float) height;

  if (width  > _WIDTH ){  width = _WIDTH; }
  if (height > _HEIGHT){ height = _HEIGHT;}

  glViewport( 0, 0, width, height );
  glMatrixMode( GL_PROJECTION );
  glLoadIdentity();
  glFrustum( -ratio*0.5, ratio*0.5, -1.0*0.5, 1.0*0.5,
	     sgl3d_near_view_volume, sgl3d_far_view_volume);
  glMatrixMode( GL_MODELVIEW );
  glLoadIdentity();
  glTranslatef( - sgl3d_view_position_x,
		- sgl3d_view_position_y,
		- sgl3d_view_position_z );
}
//============================================================================
void sgl3d_change_view_volume(double near, double far)
{
  sgl3d_near_view_volume=near;
  sgl3d_far_view_volume=far;
}
//============================================================================
void sgl3d_change_center_position( double x, double y, double z )
{
  sgl3d_view_position_x=x;
  sgl3d_view_position_y=y;
  sgl3d_view_position_z=z+sgl3d_view_length*(0.5+sgl3d_near_view_volume);
}
//============================================================================
void sgl3d_change_center_xy(double x, double y)
{
  sgl3d_view_position_x=x;
  sgl3d_view_position_y=y;
}
//============================================================================
void sgl3d_open_window(int height,int width,double length, GLfloat *Colorname)
{
  double w_red, w_green, w_blue;
  double default_near=sgl3d_near_view_volume;
  printf("default_near=%f\n",default_near);
  glutInitDisplayMode (GLUT_DOUBLE | GLUT_RGBA |GLUT_DEPTH); //double buffer */
  glutInitWindowPosition (0, 0); // corner coordinates of window
  glutInitWindowSize (height, width);  // Window size height*width    
  window = glutCreateWindow("OpenGL"); // window name "Open GL"       
  if (!window) {
    printf("Error, couldn't open window\n");
    exit(1);
  }

  sgl3d_view_length = length;
  sgl3d_near_view_volume=default_near;
  //sgl3d_view_position_z=sgl3d_view_length*(0.5+sgl3d_near_view_volume);
  sgl3d_view_position_z=5.1;
	
  //sgl3d_far_view_volume=2.0*(sgl3d_view_position_z+0.5*sgl3d_view_length);
  sgl3d_far_view_volume=sgl3d_far_view_volume;
  printf("sgl3d_view_position_z=%f\n",sgl3d_view_position_z);

  w_red   = Colorname[0];
  w_green = Colorname[1];
  w_blue  = Colorname[2];

  glClearColor(w_red, w_green, w_blue,1.0);	    // Window color 
  glClear(GL_COLOR_BUFFER_BIT|GL_DEPTH_BUFFER_BIT); // Window color setting 
  glShadeModel(GL_SMOOTH);                          // Drawing smooth picture
  sgl3d_init();		                            // Initialize 
  glutReshapeFunc(sgl3d_reshape);

}
//============================================================================
void sgl3d_loop(void func())
{
  glutDisplayFunc(func);
  glutMainLoop();
}
//============================================================================
void sgl3d_color(GLfloat *Colorname)
{
  glMaterialfv(GL_FRONT_AND_BACK,GL_AMBIENT_AND_DIFFUSE,Colorname);
}
//============================================================================
void sgl3d_color_alpha(GLfloat *Colorname,double alpha)
{
  double tmp_color_alpha=Colorname[3];
  Colorname[3]=alpha;
  glMaterialfv(GL_FRONT_AND_BACK,GL_AMBIENT_AND_DIFFUSE,Colorname);
  Colorname[3]=tmp_color_alpha;
}
//============================================================================
void sgl3d_frame(void)
{
  sgl3d_line( -1, -1, -1,   1, -1, -1,   1.0 );
  sgl3d_line( -1,  1, -1,   1,  1, -1,   1.0 );
  sgl3d_line( -1, -1,  1,   1, -1,  1,   1.0 );
  sgl3d_line( -1,  1,  1,   1,  1,  1,   1.0 );

  sgl3d_line( -1, -1, -1,  -1,  1, -1,   1.0 );
  sgl3d_line(  1, -1, -1,   1,  1, -1,   1.0 );
  sgl3d_line( -1, -1,  1,  -1,  1,  1,   1.0 );
  sgl3d_line(  1, -1,  1,   1,  1,  1,   1.0 );

  sgl3d_line( -1, -1, -1,  -1, -1,  1,   1.0 );
  sgl3d_line(  1, -1, -1,   1, -1,  1,   1.0 );
  sgl3d_line( -1,  1, -1,  -1,  1,  1,   1.0 );
  sgl3d_line(  1,  1, -1,   1,  1,  1,   1.0 );
}
//============================================================================
void sgl3d_display(void)
{
  sgl3d_frame();
  glutSwapBuffers();
}
//============================================================================
void sgl3d_clear(void)
{
  glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );
}
//============================================================================






